/**
 * BookingFormInfoStep
 * @module components/booking-form-info-step
 */
import HttpRequest from "./HttpRequest.mjs";
import Step from "./step.mjs";
import log from "../services/log.mjs";
import GoogleAutocomplete from "../services/google-autocomplete.mjs";
import PhoneNumberValidator from "../services/phone-number-validator.mjs";
import {FieldValidator} from "../services/field-validator.mjs";
import GoogleAddressValidation from "../services/google-address-validation.mjs";
import AddressAutoCompleteResult from "./AddressAutoCompleteResult.mjs";

class BookingFormInfoStep extends Step {


    endpoints = {
        countries: '/api/countries',
    };

    bookingForm = null;

    containers = {
        selectedServices: null,
        clientInfo: null
    };

    divisionId = null;

    GoogleAutocomplete = null;

    addressSuggestions = [];

    countriesLoaded = false;

    addressFieldContainer = null;

    /**
     * Selected Services
     * @type {[]}
     */
    services = [];

    HTMLElement = null;


    #hooks = {
        selectedLoaded: new CustomEvent('selectedLoaded', {detail: {step: 'info'}}),
        formFieldValidationEvent: new CustomEvent('jims-wpcb-form-field-validation', {
            bubbles: true,
            detail: {
                field: null,
                status: false,
            }
        })
    }

    #memory = {}

    constructor(bookingForm, mode = 'default') {
        super(bookingForm, mode);
        this.bookingForm = bookingForm;

        this.containerCSSSelector = `.${this.bookingForm.prefixClass}__step.step.client-info`;
    }

    init() {
        super.init();
        this.HTMLElement = this.getContainer();


        this.addressFieldContainer = this.HTMLElement.querySelector('.form-address-field');

        //jims-wpcb__step__selected_services selected_services
        this.containers.selectedServices = this.getChild(`.${this.bookingForm.prefixClass}__step__selected_services ul`);

        //jims-wpcb__step__client_info client_info
        this.containers.clientInfo = this.getChild(`.${this.bookingForm.prefixClass}__step__client_information`);

        this.load();
    }

    setSelected() {
        this.services = this.bookingForm.getSelectedServices();
        this.divisionId = this.bookingForm.getSelectedDivision();

        this.renderSelectedServices();

        //dispatch event
        this.HTMLElement.dispatchEvent(this.#hooks.selectedLoaded);
    }

    visible() {
        super.visible();

        //show submit button
        this.bookingForm.setSubmitButtonVisibility(true);
        this.bookingForm.setSubmitButtonState(true);

        //get selected services
        if(!this.hasPreselectedServices(this.bookingForm.options)){
            this.setSelected();
        }
    }

    load() {
        const options = this.bookingForm.options;

        //get selected division if step active else use the default division
        this.bookingForm.setStepTitle('Book a Service:');
        this.bookingForm.setStepTitleVisibility(true);

        this.bookingForm.setPreviousButtonState(true);
        this.bookingForm.setNextButtonState(false, true);

        if(this.hasPreselectedServices(options)){
            this.setSelected();
        }

        this.setupCountries();
        this.setupAddressAutocomplete();

        this.listen();
    }

    hasPreselectedServices(options) {
        return options.bookingFormServiceIds && options.bookingFormServiceIds.length > 0;
    }

    listen() {
        super.listen();

        this.enableValidation();

        this.HTMLElement.addEventListener('jims-wpcb-form-field-validation', (evt) => {
            this.dirty = this.bookingForm.isDirty();

            if(this.dirty){
                this.bookingForm.setSubmitButtonState(true);
            } else {
                this.bookingForm.setSubmitButtonState(false);
            }

            Object.keys(this.bookingForm.formFieldValidationStatus).forEach((name) => {
                let field = this.HTMLElement.querySelector(`[name="${name}"]`);
                if (this.bookingForm.formFieldValidationStatus[name]) {
                    field.classList.remove('error');
                } else {
                    field.classList.add('error');
                }
            });
        });

        this.HTMLElement.addEventListener('keyup', (evt) => {
            if (evt.target.nodeName === 'INPUT') {
                //if classList contains phone, prevent user using letter keys
                if (evt.target.classList.contains('phone') || evt.target.type === 'tel') {
                    evt.target.value = evt.target.value.replace(/[^0-9]/g, '');
                }
            }
        });
    }


    reset() {

    }

    renderSelectedServices() {
        this.containers.selectedServices.innerHTML = '';

        this.bookingForm.getSelectedServicesObjectList(this.services).then((services) => {
            services.forEach((service) => {
                if (this.services.includes(service.id)) {
                    let li = document.createElement('li');
                    li.innerHTML = service.name;
                    this.containers.selectedServices.appendChild(li);
                }
            });
        });
    }

    enableValidation() {
        this.addMultipleEventListener(this.HTMLElement, ['change', 'click', 'blur', 'focusout'], (evt) => {
            this.validate(evt);
        });
    }

    getFieldType(field) {
        if (field.nodeName === 'TEXTAREA') {
            return 'textarea';
        }

        if (field.nodeName === 'SELECT') {
            return 'select';
        }

        return field.type;
    }

    validate(evt) {
        let PhoneValidator = new PhoneNumberValidator(this.bookingForm.options.rest_url);
        let countryCode = this.HTMLElement.querySelector('select[name="country"]').value;
        let field = evt.target;

        if (countryCode.length === 0) {
            countryCode = this.bookingForm.options.country;
        }

        //check if field nodeName is INPUT | SELECT | TEXTAREA
        if (!['INPUT', 'TEXTAREA'].includes(field.nodeName)) {
            return;
        }

        if (field.name === 'address') {
            this.updateValidationStatus(field, false);
        }

        log.info('validation running')

        let addressValidationTime = null;

        //if target is required
        if (field.hasAttribute('required')) {
            field.classList.add('touched');
            //if field is empty
            if (field.type === 'tel') {
                PhoneValidator.validate(
                    countryCode,
                    field.value,
                ).then((response) => {
                    this.bookingForm.formFieldValidationStatus[field.name] = response.result;
                    this.HTMLElement.dispatchEvent(this.#hooks.formFieldValidationEvent);
                });
            } else {
                if (field.name === 'address' && ((this.bookingForm.options.google_address_validation && this.bookingForm.options?.google_address_validation === 0) || this.bookingForm.options?.google_address_validation === '1')) {
                    this.validateAddressField(field, countryCode);
                } else {
                    let status = FieldValidator.validate(this.getFieldType(field), field.value);
                    this.bookingForm.formFieldValidationStatus[field.name] = status;
                    this.HTMLElement.dispatchEvent(this.#hooks.formFieldValidationEvent);
                }
            }
        }
    }

    /**
     * Update the validation status of a field
     * @param field
     * @param result
     */
    updateValidationStatus(field, result) {
        //validate field name to not include square brackets
        if (field.name.includes('[') || field.name.includes(']')) {
            return;
        }

        this.bookingForm.formFieldValidationStatus[field.name] = result;
        this.HTMLElement.dispatchEvent(this.#hooks.formFieldValidationEvent);
    }

    /**
     *
     * @param field
     * @param response
     * @param messageElement
     * @param AddressValidator
     * @return {boolean}
     */
    handleAddressValidationResult(field, formattedAddress, response, messageElement, AddressValidator) {
        this.updateValidationStatus(field, response.result);
        field.value = formattedAddress;

        if (response.result === false && !response.hasOwnProperty('incomplete')) {
            let friendlyAddressParts = AddressValidator.getFriendlyAddressParts(response.data.address);

            let message = [
                'Malfomed address',
                friendlyAddressParts.length > 0 ? `Missing ${friendlyAddressParts.join(', ')}` : '',
                'Please enter a valid address'
            ].join('. ');

            messageElement.classList.remove('hidden');

            //assign message and remove line returns
            messageElement.innerHTML = message.replaceAll('\n', '');

            return false;
        }
    }

    toggleLoaderOnAddress(open = true) {

        if(open){
            this.addressFieldContainer.classList.add('field-loading');
        } else {
            this.addressFieldContainer.classList.remove('field-loading');
        }
    }

    validateAddressField(field, countryCode) {
        //check if field is focused
        if (document.activeElement === field) {
            this.updateValidationStatus(field, false);
            return;
        }

        if (field.value.length < 5) {
            return;
        }

        this.toggleLoaderOnAddress(true);
        field.classList.remove('error');
        let AddressValidator = new GoogleAddressValidation(this.bookingForm.options.google_api_key);
        let messageElement = this.HTMLElement.querySelector('.address-error-message');

        //disable country select
        this.setCountrySelectState('disabled');


        //hide message
        messageElement.classList.add('hidden');
        messageElement.innerHTML = '';

        if (this.bookingForm.googleAutocompleteIsVisible()) {
            this.observeGoogleAutocomplete();
            this.updateValidationStatus(field, false);
            return false;
        }

        //if the google address validation is passed check if the country is correct
        this.bookingForm.getAddressObject(field.value, countryCode).then((addressObject) => {
            if (addressObject.exception) {
                // field.classList.add('error');
                messageElement.classList.remove('hidden');
                messageElement.innerHTML = 'Please enter a valid address';

                this.toggleLoaderOnAddress(false);
                this.updateValidationStatus(field, false);
                return false;
            }

            let calculatedCountryCode = addressObject.country.length > 3
                ? this.bookingForm.countriesCodes.getCountryCode(addressObject.country)
                : addressObject.country;

            if (countryCode !== calculatedCountryCode) {
                messageElement.classList.remove('hidden');

                //assign message and remove line returns
                messageElement.innerHTML = `Please enter a valid address for ${this.bookingForm.countriesCodes.getCountryName(countryCode)}`;

                this.toggleLoaderOnAddress(false);
                this.updateValidationStatus(field, false);
            } else {
                let formattedAddress = AddressValidator.formatAddress(addressObject);

                AddressValidator.validateAddress({
                    "address": {
                        regionCode: countryCode,
                        addressLines: [formattedAddress]
                    }
                }).then((response) => {
                    this.toggleLoaderOnAddress(false);
                    this.handleAddressValidationResult(field, formattedAddress, response, messageElement, AddressValidator);
                }).finally(() => {
                    this.setCountrySelectState('enabled');
                });
            }
        }).catch((error) => {
            messageElement.classList.remove('hidden');
            messageElement.innerHTML = 'Please enter a valid address';
            log.info(error);

            this.toggleLoaderOnAddress(false);
            this.updateValidationStatus(field, false);
        }).finally(() => {
            this.setCountrySelectState('enabled');
        });
    }

    /**
     * @return
     */
    observeGoogleAutocomplete() {
        if (!this.bookingForm.googleAutoCompleteActive()) return;

        let observer = new MutationObserver((mutations) => {
            mutations.forEach((mutation) => {
                if (mutation.target.style.display === 'none') {
                    this.HTMLElement.querySelector('form [name=address]').dispatchEvent(new Event('change', {
                        cancelable: true,
                        bubbles: true
                    }));
                }
            });
        });

        observer.observe(document.querySelector('.pac-container'), {
            attributes: true,
            attributeFilter: ['style']
        });
    }

    setupCountries() {
        if(this.countriesLoaded) return;

        let http = new HttpRequest(this.bookingForm.options.rest_url);
        let countriesSelect = this.HTMLElement.querySelector('select[name="country"]');
        const config = this.bookingForm.options;

        if (countriesSelect === null) { return false; }

        let countriesResponse = [];
        http.get(this.bookingForm.endpoints.countries).then((response) => {
            countriesResponse = response;
        }).catch((error) => {
            log.error(error);
        }).finally(() => {
           if(countriesResponse.hasOwnProperty('length') && countriesResponse.length > 0){
               this.bookingForm.countries = countriesResponse;
               countriesSelect.innerHTML = '';
               this.bookingForm.countries.forEach((country) => {
                   let option = document.createElement('option');
                   option.value = country.symbol;
                   option.innerText = country.name;

                   if (config.country && country.symbol === config.country) {
                       option.setAttribute('selected', 'selected');
                       this.setFlag(country.symbol);
                   }

                   countriesSelect.appendChild(option);
               });

               if(this.bookingForm.countries.length > 0){
                   this.countriesLoaded = true;
               }
           }
        });

        countriesSelect.addEventListener('change', (evt) => {
            this.setFlag(evt.target.value);
            if (this.bookingForm.useGoogleAutocomplete) {
                this.GoogleAutocomplete.setCountry(evt.target.value);
            }

            //revalidate address field
            let addressField = this.HTMLElement.querySelector('input[name="address"]');
            if (addressField.value.length > 5) {
                const e = new Event("change", {
                    'bubbles': true,
                    'cancelable': true
                });
                addressField.dispatchEvent(e);
            }
        });
    }

    /**
     * Requests address autocomplete suggestions from findajim service
     * @param address
     * @param addressInput
     */
    getAddressSuggestions(address) {
        const httpRequest = new HttpRequest(this.bookingForm.options.rest_url);
        httpRequest.post(this.bookingForm.endpoints.address, {
            "address": address,
            "country_code": this.HTMLElement.querySelector('select[name="country"]').value
        }).then((response) => {
            this.addressSuggestions = response;
            this.setAddressSuggestions();
        });
    }

    /**
     * Adds options to auto-complete for address
     */
    setAddressSuggestions() {
        const autocompleteElementContainer = this.HTMLElement.querySelector(`.${this.prefixClass}__autocomplete`);
        autocompleteElementContainer.innerHTML = '';
        autocompleteElementContainer.classList.add('open');

        //populate datalist with suggestions
        this.addressSuggestions.forEach((address) => {
            let autocompleteElement = new AddressAutoCompleteResult(
                address,
                this.prefixClass,
                this.HTMLElement.querySelector('input[name="address"]'),
                autocompleteElementContainer
            );
            autocompleteElementContainer.append(autocompleteElement.render());
        });
    }

    setFlag(countryCode) {
        let countryImage = countryCode.toLowerCase();
        this.HTMLElement.querySelector('.icon-country img').setAttribute('src', `${this.bookingForm.options.wpcb_url}assets/dist/img/${countryImage}.png`);
    }

    setCountrySelectState(state = 'disabled') {
        let countrySelect = this.HTMLElement.querySelector('select[name="country"]');
        if(state === 'enabled'){
            countrySelect.removeAttribute('disabled');
            return;
        }
        countrySelect.setAttribute('disabled', 'disabled');
    }

    /**
     * Refreshes the address suggestions
     */
    setupAddressAutocomplete() {
        const addressInput = this.HTMLElement.querySelector('input[name="address"]');

        if (this.bookingForm.useGoogleAutocomplete) {
            this.GoogleAutocomplete = new GoogleAutocomplete(
                addressInput,
                this.HTMLElement,
                {
                    countryCode: this.bookingForm.options.country,
                },
                this.bookingForm.options.google_api_key
            );
        } else {
            let handler = null;
            addressInput.addEventListener('keyup', (evt) => {
                handler = setTimeout(() => {
                    clearTimeout(handler);
                    this.getAddressSuggestions(evt.target.value);
                }, 500);
            });
        }
    }

    addMultipleEventListener(element, events, callback) {
        events.forEach((event) => {
            element.addEventListener(event, callback);
        });
    }
}

export default BookingFormInfoStep;