/**
 * SettingsServiceOrdering
 */
import Service from "./service.mjs";
import JimServices from "../../services/jims-services.mjs";
import log from "../../services/log.mjs";

/**
 *  An object representing the SettingsServiceOrdering component.
 */
class SettingServiceOrdering {

    services = [];

    constructor(element, options) {
        this.element = element;
        this.options = options;

        this.api = new JimServices(options.rest_url, options.bookingFormDivisionId, options.country);
    }

    init() {
        this.loadServices();
    }

    loadServices() {
        //load services
        let services = this.api.getServiceList().then((services) => {
            const servicesList = Object.values(services);

            this.api.getOrder().then((order) => {
                if(order.data.length > 0) {
                    this.render(servicesList, order.data);
                } else {
                    this.render(servicesList);
                }
            });
        });
    }

    render(servicesList, order = []) {

        servicesList.forEach((service, index) => {
            let gravity = servicesList.length - index;

            if (order.length > 0) {
                let savedServiceOrder = order.find((savedService) => {
                    return savedService.id === service.id;
                });

                if (savedServiceOrder !== undefined) {
                    gravity = savedServiceOrder.gravity;
                }
            }

            let serviceObject = new Service(service.id, service.friendly_name, 0, '', '', gravity);
            this.services.push(serviceObject);

            serviceObject.createDomElement();
            serviceObject.update();
            this.element.querySelector('ul').appendChild(serviceObject.element);
        });

        this.setBehaviour();
    }

    setBehaviour() {
        this.services.forEach((service, index) => {
            service.onchevronClick((position) => {
                this.updateServicePosition(service, index, position);
            });

            service.valueChanged((inputElement) => {
                let value = inputElement.value;
                log.info('value :' + value, 'condition: ' + (!isNaN(value) && value >= 1 || value <= this.services.length));

                //check if the value is a number and if it is between 1 and the number of services
                if (!isNaN(value) && value >= 1 || value <= this.services.length) {
                    //find the service with the gravity property equal to the value
                    let serviceWithGravity = this.services.find((serviceItem) => {
                        return serviceItem.gravity === parseInt(value);
                    });

                    if (serviceWithGravity !== undefined) {
                        this.swapGravity(service, serviceWithGravity);
                    }
                }
            });
        });
    }

    updateServicePosition(service, index, position) {
        if (position === 1) {
            let listOfServicesWithBiggerGravity = this.services.filter((service) => {
                return service.gravity > this.services[index].gravity;
            }).sort((a, b) => {
                return a.gravity - b.gravity;
            });


            if (listOfServicesWithBiggerGravity[0] !== undefined) {
                this.swapGravity(service, listOfServicesWithBiggerGravity[0]);
            }

            // service.gravity = listOfServicesWithBiggerGravity[0] !== undefined ? listOfServicesWithBiggerGravity[0].gravity + 1 : service.gravity + 1;
            service.update();
        }

        if (position === -1) {
            let listOfServicesWithSmallerGravity = this.services.filter((service) => {
                return service.gravity < this.services[index].gravity;
            }).sort((a, b) => {
                return b.gravity - a.gravity;
            });

            if (listOfServicesWithSmallerGravity[0] !== undefined) {
               this.swapGravity(service, listOfServicesWithSmallerGravity[0]);
            }

            // service.gravity = listOfServicesWithSmallerGravity[0] !== undefined ? listOfServicesWithSmallerGravity[0].gravity - 1 : service.gravity - 1;
            service.update();
        }
    }

    swapGravity(service1, service2) {
        let gravity = service1.gravity;
        service1.gravity = service2.gravity;
        service2.gravity = gravity;

        service1.update();
        service2.update();
    }

    /**
     * Update order is triggered when the gravity property of a service is changed
     *
     * WARNING: The service order on the GUI is based on flexbox order property.
     * so if it doesn't work check the CSS styles
     */
    updateOrder() {
        //update order

        //check the gravity property of each service object and update the css order property

    }

    save() {
        let servicesObject = this.services.map((service) => service.toJsonObject());
        log.info(servicesObject);

        this.api.setOrder(servicesObject).then((response) => {
           log.info(response);
        });
    }
}

export default SettingServiceOrdering;