/**
 * Settings Form Customizer
 */
import SettingServiceOrdering from "./setting-service-ordering/setting-service-ordering.mjs";
import SettingServiceGrouping from "./setting-service-grouping/setting-service-grouping.mjs";
import SettingServiceDescriptions from "./setting-service-descriptions.mjs";
import JimsServices from "../services/jims-services.mjs";

/**
 * Settings Form Customizer
 */
class SettingsFormCustomizer {

    /**
     * Element
     * @type {null|HTMLElement}
     */
    element = null;

    /**
     * Options
     * @type {null}
     */
    options = null;

    /**
     * @type {JimsServices}
     */
    api = null;

    /**
     * Toggles
     * @type {{custom_order: *, categories_enabled: *, alphabetical_order: *, popularity_based: *}}
     */
    toggles = {
        // popularity_based: this.getElement('.jims-switch input[name=popularity_based]'),
        alphabetical_order: this.getElement('.jims-switch input[name=alphabetical_order]'),
        custom_order: this.getElement('.jims-switch input[name=custom_order]'),
        categories_enabled: this.getElement('.jims-switch input[name=categories_enabled]'),
        service_descriptions_enabled : this.getElement('.jims-switch input[name=service_descriptions_enabled]')
    };

    /**
     * Constructor
     * @param element HTMLElement
     * @param options Object
     */
    constructor(element, options) {
        this.element = element;
        this.options = options;

        this.api = new JimsServices(this.options.rest_url, this.options.bookingFormDivisionId);

        this.settingServiceOrdering = new SettingServiceOrdering( this.getElement('.jims-services-order'), options );
        this.settingServiceGrouping = new SettingServiceGrouping( this.getElement('.jims-service-panel.categories'), options );
        this.settingServiceDescriptions = new SettingServiceDescriptions( this.getElement('.jims-service-panel.service-descriptions'), options, this );
    }

    /**
     * Initialize the settings form customizer
     */
    init() {
        this.settingServiceOrdering.init();
        this.settingServiceGrouping.init();
        this.settingServiceDescriptions.init();


        this.api.getOptions().then((response) => {

            if(Object.keys(response.data).length > 0) {
                let savedOptions = response.data;
                console.log(savedOptions);
                
                // this.toggles.popularity_based.checked = savedOptions.popularity_based;
                this.toggles.alphabetical_order.checked = savedOptions.alphabetical_order;
                this.toggles.custom_order.checked = savedOptions.custom_order;
                this.toggles.categories_enabled.checked = savedOptions.categories_enabled;
                this.toggles.service_descriptions_enabled.checked = savedOptions.service_descriptions_enabled;

                this.setOrderingState(this.toggles.custom_order.checked);
                this.setGroupingState(this.toggles.categories_enabled.checked);
                this.setServiceDescriptionState(this.toggles.service_descriptions_enabled.checked);
            }

            this.setToggleBehavior();
        });


        this.element?.addEventListener('submit', (e) => {
            e.preventDefault();
            this.save();

            return false;
        });
    }

    /**
     * Save the settings
     */
    save() {
        let submit = this.element.querySelector('input[type=submit]');

        submit.classList.add('disabled');
        //Create the settings object without the independent setting components
        let data = {
            // popularity_based: this.toggles.popularity_based?.checked,
            alphabetical_order: this.toggles.alphabetical_order?.checked,
            custom_order: this.toggles.custom_order?.checked,
            categories_enabled: this.toggles.categories_enabled?.checked,
            service_descriptions_enabled: this.toggles.service_descriptions_enabled?.checked
        };

        //save individual setting components
        this.settingServiceOrdering.save();
        this.settingServiceGrouping.save();
        this.settingServiceDescriptions.save();

        //save the settings
        this.api.setOptions(data).then((response) => {
            this.element.addEventListener('change', (e) => {
                e.preventDefault();
                submit.classList.remove('disabled');

                return false;
            });
        });
    }

    /**
     * Set the behavior of the toggles
     */
    setToggleBehavior() {
        // if popularity based is enabled, disable alphabetical order and if alphabetical order is enabled, disable popularity based
        // if popularity based or alphabetical order is enabled, disable custom ordering

        this.toggles.alphabetical_order.addEventListener('change', () => {
            if ( this.toggles.alphabetical_order.checked ) {
                this.toggles.custom_order.checked = false;
            }

            //disable custom ordering panel
            this.setOrderingState( this.toggles.custom_order.checked );
        });

        this.toggles.custom_order.addEventListener('change', () => {
            if ( this.toggles.custom_order.checked ) {
                this.toggles.alphabetical_order.checked = false;
            }
            this.setOrderingState( this.toggles.custom_order.checked );
        });

        this.toggles.categories_enabled.addEventListener('change', () => {
            this.setGroupingState( this.toggles.categories_enabled.checked );
        });

        this.toggles.service_descriptions_enabled.addEventListener('change', () => {
            this.setServiceDescriptionState( this.toggles.service_descriptions_enabled.checked );
        });
    }

    /**
     * Set the state of the grouping toggle
     * @param state
     */
    setGroupingState(state){
        let categoryPanel = this.getElement('.jims-service-panel.categories');

        if ( state ) {
            categoryPanel.classList.remove('disabled');
        } else {
            categoryPanel.classList.add('disabled');
        }
    }

    /**
     * Set the state of the ordering toggle
     * @param state
     */
    setOrderingState(state){
        let orderingPanel = this.getElement('.jims-services-order');

        if ( state ) {
            orderingPanel.classList.remove('disabled');
        } else {
            orderingPanel.classList.add('disabled');
        }
    }

    setServiceDescriptionState(state){
        let serviceDescriptionPanel = this.getElement('.jims-service-panel.service-descriptions');

        if ( state ) {
            serviceDescriptionPanel.classList.remove('disabled');
        } else {
            serviceDescriptionPanel.classList.add('disabled');
        }
    }

    /**
     * Get an element by selector
     * @param selector
     * @return {*}
     */
    getElement(selector) {
        return document.querySelector(selector);
    }
}

export default SettingsFormCustomizer;