<?php

namespace JimsCustomerBooking\Database;

use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Jims_Plugin;

/**
 * Class Database
 */
class Database
{
    public static $queries_to_run = [];

    /**
     * @var array $migrations
     * List of migrations to run
     * a migration is a function that creates or modifies a table
     * the migrations array has the version as the key, and the name of the migration
     * function as the value
     *
     * Every migration function name should start with 'migration_'
     */
    public static $migrations = [
        '3.1.03' => 'migration_add_error_code_column',
        '3.3.01' => 'migration_add_division_id_column'
    ];

    public static function setupOrUpdateDatabase($current_version = '0.0.0')
    {
        error_log('setupOrUpdateDatabase', 0);
        self::setup();

        self::update($current_version);

        $result = self::run();
    }

    public static function run()
    {
        return self::runSQL(self::$queries_to_run, true);
    }

    /**
     * @return void
     */
    public static function setup(): void
    {

        if(Database::tableExists('customer_bookings')) return;

        self::createTable([
            'id mediumint(9) NOT NULL AUTO_INCREMENT',
            'booking_id varchar(255) NOT NULL',
            'division_id int(255) NULL',
            'title varchar(255) NULL',
            'data longtext NOT NULL',
            'status varchar(255) NOT NULL',
            'error_code varchar(255) NULL',
            'created_at datetime DEFAULT \'0000-00-00 00:00:00\' NOT NULL',
            'updated_at datetime DEFAULT \'0000-00-00 00:00:00\' NOT NULL',
            'PRIMARY KEY  (id)'
        ], 'customer_bookings');
    }

    /**
     * @param $oldVersion
     * @return array
     */
    public static function update($oldVersion)
    {
        $output = [];

        if ($oldVersion === '0.0.0') {
            $oldVersion = '3.1.0'; //set the first version with migration system
        }

        //run all the migration functions that are greater than the old version
        foreach (self::$migrations as $version => $migration) {

            if (StringHelper::compareVersionStrings($version, $oldVersion)) {
                array_push($output, [
                    'version' => $version,
                    'result' => call_user_func_array(self::class . '::' . $migration, [])
                ]);
            }
        }

        return $output;
    }


    /**
     * Creates a table in the database, from an array of columns
     *
     * @param array $columns
     * @param $table_name
     * @return bool
     */
    public static function createTable(array $columns, $table_name)
    {
        $bookings_table_name = '';
        $sql = self::getCreateTableSQL($columns, $table_name, $bookings_table_name);

        self::addQueryToRun($sql);

        return true;
    }

    /**
     * @param array $columns
     * @param $table_name
     * @param $bookings_table_name
     * @return false|string
     */
    public static function getCreateTableSQL(array $columns, $table_name, &$bookings_table_name)
    {
        global $wpdb;

        if (count($columns) == 0) {
            return false;
        }

        $charset_collate = $wpdb->get_charset_collate();

        $bookings_table_name = self::getTableName($table_name);

        $columns = implode(',', $columns);
        return "CREATE TABLE IF NOT EXISTS $bookings_table_name ($columns) $charset_collate";
    }

    /**
     * @return bool|int|\mysqli_result|resource|null
     */
    public static function deleteAllTransients()
    {
        global $wpdb;

        $table_name = $wpdb->prefix . 'options';
        $plugin_prefix = Jims_Plugin::PREFIX;

        $sql = "DELETE FROM $table_name WHERE `option_name` like '%transient%' and `option_name` like '%$plugin_prefix%';";

        return $wpdb->query($sql);
    }

    /**
     * @param $sql , string|string[]
     * @return array
     */
    public static function runSQL($queries, $use_dbDelta = true)
    {
        $result = [];

        if ($use_dbDelta) {
            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
            $result = dbDelta($queries);

            if (!is_array($result)) {
                $result = [$result];
            }
        }

        //import the global wpdb object
        global $wpdb;

        if (count($result) < count($queries) + 1) {
            $result = array_map(function ($query) use ($wpdb) {
                return $wpdb->query($query);
            }, $queries);
        }

        return $result;
    }

    /**
     * @return bool
     */
    public static function deleteDatabase()
    {
        global $wpdb;

        $table_name = self::getTableName();

        $sql = "DROP TABLE IF EXISTS $table_name";

        return $wpdb->query($sql);
    }

    /**
     * @param \wpdb $wpdb
     * @return string
     */
    protected static function getTableName($name = 'customer_bookings'): string
    {
        global $wpdb;
        return implode('_', [$wpdb->prefix . Jims_Plugin::PREFIX, $name]);
    }

    /**
     * @param array $query
     * @return void
     */
    public static function addQueryToRun(string $query): void
    {
        self::$queries_to_run[] = $query;
    }

    public static function tableExists($table_name)
    {
        global $wpdb;

        $table_name = self::getTableName($table_name);

        $sql = "SHOW TABLES LIKE '$table_name'";

        return $wpdb->get_var($sql) == $table_name;
    }

    public static function dbLog($message)
    {
        error_log(json_encode($message), 0, '/var/www/jims-wp-default.local/wp-content/plugins/jims-wpcb/db.log');
    }


    //Migration Functions

    /**
     * @return array|false|int
     */
    public static function migration_add_error_code_column()
    {
        $table_name = self::getTableName('customer_bookings');

        //add error_code column after status column
        $sql = "ALTER TABLE $table_name ADD COLUMN error_code varchar(255) NULL AFTER status";

        self::addQueryToRun($sql);

        return $sql;
    }

    public static function migration_add_division_id_column()
    {
        $table_name = self::getTableName('customer_bookings');

        //add error_code column after status column
        $sql = "ALTER TABLE $table_name ADD COLUMN division_id int(255) NULL AFTER booking_id";

        self::addQueryToRun($sql);

        return $sql;
    }
}