<?php

namespace JimsCustomerBooking\Helpers;

/**
 * This class offers helper methods for time
 */
class TimeHelper extends Helper
{
    /**
     * This function converts a date to UTC
     *
     * @param string $date
     * @param string $timezone , default is UTC, other options are 'Europe/London', Melbourne/Australia
     * @param string $format_from
     * @param string $format_to
     * @param string $returnType
     * @return string|\DateTime
     */
    public static function convertToUTC($date, $timezone = 'UTC', $format_from = 'd/m/Y', $format_to = 'd/m/Y', $returnType = 'string')
    {

        if (empty($date)) return $date;

        if (is_numeric($date)) {
            $date = date($format_from, $date);
        }

        if (!self::isValidTimeZone($timezone)) {
            $timezone = 'UTC';
        }

        try {
            $date = \DateTime::createFromFormat( $format_from, $date, new \DateTimeZone( $timezone ) );
            // Set the time to 00:00:00 in the local timezone
            $date->setTime(0, 0, 0);

            // Get the offset in seconds between UTC and the local timezone
            $offset = $date->getOffset();

            // Remove this offset from the date
            $date->setTimezone( new \DateTimeZone('UTC') );

            $date->add( \DateInterval::createFromDateString( $offset . 'hours' ) );

            // Add the offset back into the date object
            return $returnType == 'string' ? $date->format( $format_to ) : $date;
        } catch (\Exception $e) {
            error_log( $e->getMessage() );
        }

        return $date;
    }

    /**
     * @param $date
     * @param $timezone
     * @param $format_from
     * @param $format_to
     * @param $returnType
     * @return string|\DateTime
     */
    public static function convertUTCToLocal($date, $timezone, $format_from = 'd/m/Y H:i:s', $format_to = 'd/m/Y H:i:s', $returnType = 'string'){
        if (empty($date)) return $date;

        if (is_numeric($date)) {
            $date = date($format_from, $date);
        }

        if (!self::isValidTimeZone($timezone)) {
            $timezone = 'UTC';
        }

        try {
            $date = \DateTime::createFromFormat( $format_from, $date, new \DateTimeZone( 'UTC' ) );

            // Get the offset in seconds between UTC and the local timezone
            $offset = $date->getOffset();

            // Remove this offset from the date
            $date->setTimezone( new \DateTimeZone($timezone) );

            // Add the offset back into the date object
            return $returnType == 'string' ? $date->format( $format_to ) : $date;
        } catch (\Exception $e) {
            error_log( $e->getMessage() );
        }

        return $date;
    }

    /**
     * @param string $timezone
     * @return string
     */
    public static function getTimezoneOffset($timezone = 'UTC')
    {
        $date = new \DateTime('now', new \DateTimeZone($timezone));
        return $date->format('P');
    }

    /**
     * @param $countryCode
     * @return string
     */
    public static function getTimezoneFromCountryCode($countryCode)
    {
        $timezone = '';
        switch ($countryCode) {
            case 'AU':
                $timezone = 'Australia/Melbourne';
                break;
            case 'NZ':
                $timezone = 'Pacific/Auckland';
                break;
            case 'CA':
                $timezone = 'America/Toronto';
                break;
            case 'GB':
                $timezone = 'Europe/London';
                break;
            case 'US':
                $timezone = 'America/New_York';
                break;
            default:
                $timezone = 'UTC';
                break;
        }
        return $timezone;
    }


    /**
     * Returns the timezone as UTC+{{offset}}
     * @param $timezone_name
     * @return string
     */
    public static function getTimezoneOffsetDBString($timezone_name){
        $timezone = new \DateTimeZone($timezone_name);
        $offset = $timezone->getOffset(new \DateTime("now", $timezone));
        $offset = $offset / 60 / 60;

        return ($offset > 0 ? '+' : '') . $offset .':00';
    }

    /**
     * @param $date
     * @param $countryCode
     * @param string $format
     * @return string
     */
    public static function convertDateToTimezoneFromCountryCode($date, $countryCode)
    {
        $timezone = self::getTimezoneFromCountryCode($countryCode);
        return self::convertToUTC($date, $timezone);
    }

    public static function isValidTimeZone($timezone)
    {
        try {
            new \DateTimeZone($timezone);
            return true;
        } catch (\Exception $e) {
            return false;
        }
    }
}