<?php

namespace JimsCustomerBooking\Integrations;

use JimsCustomerBooking\Helpers\Http;

/**
 * Class SlackNotification
 */
class SlackNotification extends ANotification
{
    /**
     * @var string $service
     */
    protected string $service = 'slack';

    /**
     * @var array $messages
     */
    protected array $messages = [];


    /**
     * @var string $webhook
     */
    protected string $webhook = '';

    public function __construct(string $webhook, $vars = [], $test_mode = false)
    {
        $this->webhook = $webhook;
        $this->setupNotificationMessages($vars);
        $this->test_mode = $test_mode;
    }


    /**
     * @param string $type
     * @return bool
     */
    public function send(string $type = self::NOTIFICATION_TYPE_NOTIFICATION)
    {
        //send notification to slack
        if ($this->test_mode) return $this->getMessage($type);

        $this->setServiceResponse(Http::Post($this->webhook, $this->getMessage($type)));

        //check if slack responded with ok
        if (is_array($this->getServiceResponse()) && !empty($this->getServiceResponse()) && $this->hasResponseProperty()) {
            return $this->isOK() ?? false;
        }

        return false;
    }

    /**
     * @param $vars
     * @return void
     */
    public function setupNotificationMessages($vars = [])
    {
        $notification = __( '{{icon}} *Booking ID: {{booking_id}} - {{status}} - Code 0* - on {{website}}');
        $info = __('{{icon}} *Booking ID: {{booking_id}} - {{status}} - Code 0* - on {{website}}');
        $error = __('{{icon}} *Booking ID: {{booking_id}} - {{status}} - Code {{error_code}}* - on {{website}}');

        //todo: add bug icon in front of the normal icon in order to differentiate between debug mode and normal mode


        //add icon and status to vars
        if(isset($vars['error_code'])) {
            $error_code = (int) $vars['error_code'];

            //set calculated vars
            $vars['icon'] = $this->getIcon($error_code);
            $vars['status'] = $this->getStatus($error_code);
        }

        //build block messages for slack
        $this->messages = [
            self::NOTIFICATION_TYPE_NOTIFICATION => [
                'text' => $this->replaceVars($notification, $vars),
                'blocks' => [
                    [
                        'type' => 'section',
                        'text' => [
                            'type' => 'mrkdwn',
                            'text' => $this->replaceVars($notification, $vars)
                        ]
                    ]
                ]
            ],
            self::NOTIFICATION_TYPE_INFO => [
                'text' => $this->replaceVars($info, $vars),
                'blocks' => [
                    [
                        'type' => 'section',
                        'text' => [
                            'type' => 'mrkdwn',
                            'text' => $this->replaceVars($info, $vars)
                        ]
                    ]
                ]
            ],
            self::NOTIFICATION_TYPE_ERROR => [
                'text' => $this->replaceVars($error, $vars),
                'blocks' => [
                    [
                        'type' => 'section',
                        'text' => [
                            'type' => 'mrkdwn',
                            'text' => $this->replaceVars($error, $vars)
                        ]
                    ]
                ]
            ]
        ];
    }

    /**
     * @param $error_code
     * @return string
     */
    public function getIcon($error_code){
        $error_code = (int) $error_code;

        $icons = [
            0 => ':white_check_mark:',
            1 => ':no_pedestrians:',
            2=> ':file_cabinet:',
            3=> ':no_pedestrians:',
            4=> ':house:',
            5=> ':x:',
            6=> ':x:',
            7=> ':test_tube:',
            8=> ':file_cabinet:',
            9=> ':x:',
            10=> ':x:'
        ];

        return isset($icons[$error_code]) ? $icons[$error_code] : ':x:';
    }

    /**
     * @param $error_code
     * @return string
     */
    public function getStatus($error_code){
        $error_code = (int) $error_code;

        $statuses = [
            0 => 'Successful',
            1 => 'Unserviced',
            2=> 'Duplicated',
            3=> 'Unserviced',
            4=> 'Invalid Address',
            5=> 'Failed',
            6=> 'Failed',
            7=> 'Test',
            8=> 'Duplicated',
            9=> 'Failed',
            10=> 'Failed'
        ];

        return isset($statuses[$error_code]) ? $statuses[$error_code] : 'Failed';
    }

    /**
     * @param $message
     * @param array $vars
     * @return string|string[]
     */
    protected function replaceVars($message, $vars = []): array|string
    {
        //replace vars in messages
        //for each var in vars, replace the key with the value in the message
        foreach ($vars as $key => $value) {
            if(isset($vars[$key]))
            $message = str_replace('{{' . $key . '}}', $value, $message);
        }

        return $message;
    }

    /**
     * @return bool
     */
    protected function hasResponseProperty(): bool
    {
        return isset($this->getServiceResponse()['result']) && !empty($this->getServiceResponse()['result']);
    }

    /**
     * @return mixed
     */
    protected function isOK(): mixed
    {
        return $this->getServiceResponse()['result'] == 'ok';
    }
}