<?php

namespace JimsCustomerBooking\RestfulAPI;

use JimsCustomerBooking\Jims_Plugin;
use JimsCustomerBooking\Services\TokenManager;

if (!class_exists ('JimsCustomerBooking\RestfulPI\RestController')) {

    /**
     * Class RestController
     *
     * Instructions how to add a restful controller using this class
     * 1. Create a class that extends this class
     * 2. Add a route to the class
     * 3. Add a method to the class
     *
     *
     * @package JimsCustomerBooking\RestfulAPI
     */
    class RestController extends \WP_REST_Controller
    {

        /**
         * Plugin's API namespace
         *
         * @var string
         */
        protected $namespace = 'jims-wpcb';

        /**
         * Plugin's API version
         *
         * @var string
         */
        protected $version = '1';

        /**
         * Plugin's API default route
         *
         * @var string
         */
        protected $route = 'default';

        /**
         * ApiController constructor.
         */
        public function __construct()
        {
            $this->key = Jims_Plugin::SECKEY;
            $this->register_routes ();
        }

        /**
         * @return string
         */
        protected function get_namespace()
        {
            return "{$this->namespace}/v{$this->version}";
        }

        /**
         * @param null $name
         *
         * @return string
         */
        protected function get_route($name = null)
        {
            $route = $name ?: $this->route;
            return '/' . $route;
        }

        /**
         * @param $data
         * @param int $http_code
         *
         * @return \WP_REST_Response
         */
        protected function response($data, $http_code = 200)
        {
            return new \WP_REST_Response($data, $http_code);
        }


        /**
         * @return \WP_REST_Response
         */
        protected function emptyResponse()
        {
            return $this->response ([]);
        }

        /**
         * @param $param_name
         * @param $custom_message
         * @return array
         */
        protected function invalidRequest($param_name = false, $custom_message = null)
        {
            $message = "Invalid Request" . ($param_name == false ? '' : ": Missing or invalid parameter {$param_name}");
            if (is_null ($custom_message)) {
                return [
                    'status' => 404,
                    'message' => $message
                ];
            }
            return [
                'status' => 404,
                'message' => $custom_message
            ];
        }

        /**
         * @param $custom_message
         * @return array
         */
        protected function successResponse($custom_message = null)
        {
            return [
                'status' => 200,
                'message' => 'Your action has successful'
            ];
        }

        /**
         * @param $custom_message
         * @return array
         */
        protected function failedResponse($custom_message = null)
        {
            return [
                'status' => 500,
                'message' => 'Your action failed'
            ];
        }

        /**
         * @param $request \WP_REST_Request
         * @return bool
         */
        public function checkPermissions($request)
        {
            $allowed_tokens = [
                'bypass', // bypass token
            ];

            if ($request->has_param ('token')) {
                $token = $request->get_param ('token');

                if (in_array ($token, $allowed_tokens)) return true;


                if(TokenManager::verify($this->key, $token)) return true;

                // verify token
                if (wp_verify_nonce ($request->get_param ('token'), 'wp_rest')) {
                    return true;
                }
            }

            // if request has no token, return false
            return false;
        }

        protected function getWordpressRestURL($route = null)
        {
            return $route = null ? get_rest_url (null, $this->get_namespace ()) : get_rest_url (null, $this->get_namespace () . $route);
        }

        protected function transient_exists($transient_name)
        {
            return get_transient($transient_name) !== false;
        }

        protected function stored_response_is_empty($response)
        {
            //check if response is empty or the length of the response is less than 2
            return empty($response) || ( is_string ($response) && strlen($response) < 2 );
        }

        protected function getRequestParam($request, $param_name, $default = null)
        {
            return $request->get_param ($param_name) ?? $default;
        }

        protected function register_rest_route($namespace, $routes, $args = array(), $override = false)
        {
            if(!empty($routes)) {
                foreach ($routes as $route) {
                    register_rest_route ($namespace, $route, $args, $override);
                }
            }
        }
    }
}