<?php

namespace JimsCustomerBooking\RestfulAPI;

use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Services\DivisionService;

if (!class_exists('JimsCustomerBooking\RestfulPI\ServiceController')) {
    class ServiceController extends RestController
    {

        /**
         * @var string
         */
        protected $route = 'service';


        public function register_routes()
        {

            $this->register_rest_route($this->get_namespace(), [
                $this->get_route() . '/divisions',
                $this->get_route() . '/division',
            ], [
                [
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_divisions'),
                    'permission_callback' => '__return_true',
                ]
            ]);

            $this->register_rest_route($this->get_namespace(), [$this->get_route() . '/division/services'], [
                [
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_services'),
                    'permission_callback' => '__return_true',
                ]
            ]);

            $this->register_rest_route($this->get_namespace(), [$this->get_route() . '/division/icons'], [
                [
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_icons'),
                    'permission_callback' => '__return_true',
                ]
            ]);

            $this->register_rest_route($this->get_namespace(), [$this->get_route() . '/countries'], [
                [
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_countries'),
                    'permission_callback' => '__return_true',
                ]
            ]);
        }

        /**
         * @param $request
         * @return \WP_REST_Response
         * @throws \Exception
         */
        public function get_divisions($request)
        {
            $service = new DivisionService();
            $countryCode = $this->getRequestParam($request, 'country_code', 'AU');
            return $this->response($service->getDivisions(true, $countryCode));
        }

        /**
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function get_services($request)
        {
            $countryCode = $this->getRequestParam($request, 'country_code', 'AU');
            $serviceListName = $this->getRequestParam($request, 'service_list', '');
            $service = new DivisionService();

            if ($request->has_param('division_id') && (int)$request->get_param('division_id') > 0) {
                try {
                    $payload = $service->getServices(false, $countryCode, $serviceListName);
                    $customize_options = OptionHelper::get('customer_booking_customizer_options', []);

                    $filtered = array_filter($payload, function ($service) use ($request) {
                        return $service['group_id'] == $request->get_param('division_id');
                    });

                    $use_grouping = $request->get_param('use_grouping');
                    if ($use_grouping === '1') {
                        $filtered = $this->group_services($filtered, $customize_options);
                    }

                    if($serviceListName === '') { //disable custom order if custom service list is enabled
                        //if custom order is enabled
                        $custom_order_enabled = isset($customize_options['custom_order']) && $customize_options['custom_order'] == true;
                        if ($custom_order_enabled) {
                            $custom_order = OptionHelper::get('customer_booking_service_order', []);
                            $filtered = $this->orderServices($custom_order, $filtered, !empty($use_grouping));
                        }
                    }
                } catch (\Exception $e) {
                    return $this->response([]);
                }

                return $this->response($filtered);
            }

            return $this->response($service->getServices(false, $countryCode));
        }

        /**
         * @param $request
         * @return \WP_REST_Response
         * @throws \JsonException
         */
        public function get_countries($request)
        {
            $service = new DivisionService();
            return $this->response($service->getCountries());
        }

        /**
         * @param $filtered
         * @param $customize_options
         * @return array
         */
        public function group_services($filtered, $customize_options)
        {
            $groups = OptionHelper::get('customer_booking_service_groups', []);

            //get if service description is enabled
            $service_description_enabled = isset($customize_options['service_descriptions_enabled'])
                && $customize_options['service_descriptions_enabled'] == true;

            $filtered = $this->addDescriptions($service_description_enabled, $filtered);

            //get groups
            $grouped = [];
            $grouped_ids = [];
            foreach ($groups as $group) {
                if (!empty($group['services'])) {
                    $group['services'] = $this->getGroupServices($filtered, $group['services']);
                    $grouped[] = $group;
                }
            }

            //remove grouped services from filtered
            $filtered = array_filter($filtered, function ($service) use ($grouped_ids) {
                return !in_array($service['id'], $grouped_ids);
            });

            //create a custom group for the remaining services
            if (!empty($filtered)) {
                $grouped[] = $this->createRestGroup($filtered);
            }

            return $grouped;
        }

        /**
         * @param $services_order
         * @param $filtered
         * @return array
         */
        protected function orderServices($services_order, $filtered, bool $is_grouped)
        {
            if ($is_grouped) {
                foreach ($filtered as $index => $group) {
                    usort($group['services'], function ($a, $b) use ($services_order) {
                        $a_index = array_search($a['id'], array_column($services_order, 'id'));
                        $b_index = array_search($b['id'], array_column($services_order, 'id'));

                        return ($services_order[$a_index]['gravity'] > $services_order[$b_index]['gravity']) ? -1 : 1;
                    });

                    $filtered[$index]['services'] = $group['services'];

                    //sum the gravity of the services in the group
                    $gravity_sum = array_sum(array_map(function ($service) use ($services_order) {
                        $index = array_search($service['id'], array_column($services_order, 'id'));
                        return $services_order[$index]['gravity'];
                    }, $group['services']));


                    if (!(isset($filtered[$index]['ignore_order']) && $filtered[$index]['ignore_order'] === 1)) {
                        $filtered[$index]['order'] = $gravity_sum / count($group['services']);
                    }
                }
            }

            usort($filtered, function ($a, $b) use ($services_order) {
                $a_index = array_search($a['id'], array_column($services_order, 'id'));
                $b_index = array_search($b['id'], array_column($services_order, 'id'));

                return ($services_order[$a_index]['gravity'] > $services_order[$b_index]['gravity']) ? -1 : 1;
            });

            return $filtered;
        }

        protected function getDescription($service, $service_descriptions)
        {
            $descriptions = array_values(array_filter($service_descriptions, function ($description) use ($service) {
                return $description['id'] == $service['id'];
            }));

            return array_shift($descriptions);
        }

        /**
         * @param bool $service_description_enabled
         * @param $filtered
         * @return mixed
         */
        protected function addDescriptions(bool $service_description_enabled, $filtered): mixed
        {
            if ($service_description_enabled) {
                //get descriptions
                $service_descriptions = OptionHelper::get('customer_booking_service_descriptions', []);

                //populate filtered with descriptions
                $filtered = array_map(function ($service) use ($service_descriptions) {
                    //find description
                    $description = $this->getDescription($service, $service_descriptions);
                    $service['description'] = $description['description'] ?? '';

                    return $service;
                }, $filtered);
            }
            return $filtered;
        }

        /**
         * @param mixed $filtered
         * @param $services
         * @return array|null[]
         */
        public function getGroupServices(mixed &$filtered, $services): array
        {
            return array_map(function ($service) use (&$filtered) {

                $service = array_filter($filtered, function ($filtered_service) use ($service) {
                    return $filtered_service['id'] == $service['id'];
                });

                $service = array_shift($service);
                $grouped_ids[] = $service['id'];

                //remove service from filtered
                $filtered = array_filter($filtered, function ($filtered_service) use ($service) {
                    return $filtered_service['id'] != $service['id'];
                });

                return $service;
            }, $services);
        }

        /**
         * @param $request \WP_REST_Request
         * @return \WP_REST_Response
         */
        public function get_icons(\WP_REST_Request $request): \WP_REST_Response
        {
            $service = new DivisionService();
            return $this->response($service->getDivisionIcons());
        }

        /**
         * @param mixed $filtered
         * @return array
         */
        public function createRestGroup(mixed $filtered, $name = 'Other', $description = ''): array
        {
            return [
                'order' => 0,
                'description' => $description,
                'ignore_order' => 1,
                'name' => $name,
                'services' => $filtered
            ];
        }

    }
}