<?php

namespace JimsCustomerBooking\Services;

use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Helpers\Http;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Jims_Plugin;
use JimsCustomerBooking\Validators\JsonValidator;
use JimsGroup\Api;
use JimsGroup\Apis\CustomerBooking;
use JimsGroup\Facade\Authenticate;

if (!class_exists('\JimsCustomerBooking\Services\DivisionService')) {
    /**
     * TODO: Update this to use FMS API instead of internal.jims.net
     */
    class DivisionService extends Service
    {

        private $divisionsLink = JIMS_INTERNAL_URL . '/api/division/all';

        private $countriesLink = JIMS_INTERNAL_URL . '/api/division/countries/active/all';

        protected $default_countries = [
            [
                'name' => 'Australia',
                'symbol' => 'AU'
            ],
            [
                'name' => 'New Zealand',
                'symbol' => 'NZ'
            ],
            [
                'name' => 'United Kingdom',
                'symbol' => 'UK'
            ],
            [
                'name' => 'Canada',
                'symbol' => 'CA'
            ]
        ];

        /**
         * @var Api|null $Api
         */
        private $Api = null;

        public function __construct()
        {
            $this->Api = new Api();
        }

        /**
         * @return array|mixed
         * @throws \Exception
         */
        public function call(): mixed
        {
            switch ($_GET['part']) {
                case 'divisions':
                    return $this->getDivisions();
                case 'services':
                    return $this->getServices();
            }

            return [];
        }

        /**
         * @param $link
         * @return mixed
         * @throws \Exception
         */
        public function getDivisions($short = true, $countryCode = 'AU'): mixed
        {
            $output = [];
            if (false === ($value = get_transient(StringHelper::GetPrefixed('divisions' . '_' . $countryCode)))) {
                $customDivisions = $this->getCustomDivisions();

                if (empty($customDivisions)) {
                    $fmsInfo = $this->Api->getFMSInfoApi();
                    $response = $fmsInfo->getDivisions('json');
                } else {
                    $response = $customDivisions;
                }

                set_transient(StringHelper::GetPrefixed('divisions'), $response, 5 * DAY_IN_SECONDS);
                $output = json_decode($response);
            } else {
                $output = json_decode($value);
            }

            //filter out empty divisions
            $filtered_output = $this->filterOutEmptyDivisions($countryCode, $output);
            if (count($filtered_output) > 0) {
                $output = $filtered_output;
            }

            if ($short && !empty($output)) {
                $icons = $this->getDivisionIcons();

                if(false === ($short_output = get_transient(StringHelper::GetPrefixed('divisions_short_output' . '_' . $countryCode)))) {
                    $short_output = [];
                    foreach ($output as $division) {
                        if (empty($division->name)) continue;
                        $short_output[] = [
                            'id' => $division->id,
                            'name' => strtoupper($division->name),
                            'icon' => !empty($icons[$division->id]['icon']) ? $icons[$division->id]['icon'] : JIMS_CBP_URL . 'assets/dist/img/jimsgroup.png',
                        ];
                    }

                    set_transient(StringHelper::GetPrefixed('divisions_short_output' . '_' . $countryCode), $short_output, 5 * HOUR_IN_SECONDS);
                }

                return $short_output;
            }

            return $output;
        }

        /**
         * @param $division_id
         * @return array|null
         */
        public function getDivisionById($division_id): array|null
        {
            $divisions = $this->getDivisions(true);
            $division = array_filter($divisions, function ($item) use ($division_id) {
                return $item->id == $division_id;
            });

            return array_shift($division);
        }

        /**
         * @return array|mixed
         */
        public function getServices($format = false, $countryCode = 'AU', $customServiceListName = ""): mixed
        {
            $key = 'services_' . $countryCode;
            if ($customServiceListName === '') {
                $key .= '_all';
            } else {
                $key .= '_' . $customServiceListName;
            }

            $transientName = StringHelper::GetPrefixed($key);

            if (false === ($output = get_transient($transientName))) {

                $customerBooking = new CustomerBooking();
                $customerBooking->setMode($this->getMode());
                $customerBooking->setAuthToken(Jims_Plugin::FMS_API_KEY);
                $customerBooking->setHeaderAuthToken(
                    $this->getFMSAuthToken()
                );

                try {
                    // get initial booking request
                    $output = $customerBooking->getInitialBookingRequest([
                        'countryCode' => $countryCode,
                        'serviceListName' => $customServiceListName,
                    ]);

                    if(isset($output['Message'])){
                        throw new \RuntimeException($output['Message']);
                    } else {
                        // set transient
                        set_transient( $transientName, $output, 1 * HOUR_IN_SECONDS );
                    }
                } catch (\Exception $e) {
                    throw new \RuntimeException($e->getMessage());
                }
            }

            return $format === true ? $this->formatServices($output) : $output;
        }

        /**
         * @param string $return_type , object|array
         * @return array|mixed
         * @throws \JsonException
         */
        public function getCountries(string $return_type = 'object'): array
        {
            if (false === ($result = get_transient(StringHelper::GetPrefixed('countries')))) {
                [$result, $headers, $error] = Http::Get($this->countriesLink);

                //if there is an error, return the default countries
                if (!StringHelper::isJson($result) || empty($result)) {
                    $result = json_encode($this->default_countries, JSON_THROW_ON_ERROR);
                }

                set_transient(StringHelper::GetPrefixed('countries'), $result, 1 * DAY_IN_SECONDS);
            }

            return json_decode($result, $return_type === 'array', 512, JSON_THROW_ON_ERROR);
        }

        /**
         * @param $services
         * @return array|array[]
         */
        public function formatServices($services): array
        {

            return array_map(function ($service) {
                if(!is_array($service)){
                    $service = (array)$service;
                }

                return [
                    'name' => $service['friendly_name'],
                    'system_name' => $service['name'],
                    'id' => $service['id'],
                    'division_id' => $service['group_id']
                ];
            }, $services);
        }


        protected function getCustomDivisions()
        {
            if (!Http::makeTestRequest($this->divisionsLink)) {
                return null;
            }

            list($result, $headers, $error) = Http::Get($this->divisionsLink);

            if ($error) return null;

            return $result;
        }

        /**
         * @return array|array[]|mixed
         */
        public function getDivisionIcons(): mixed
        {
            $fmsInfo = $this->Api->getFMSInfoApi();
            $auth = $this->getAuth();
            $fmsInfo->setHeaderAuthToken($auth->CustomerBookingAccessToken()[1]);

            $transientName = StringHelper::GetPrefixed('fms_division_icons');
            if (false === ($requestResponse = get_transient($transientName))) {
                $requestResponse = $fmsInfo->getDivisionIcons();

                uasort($requestResponse, static function ($a, $b) {
                    return $a['id'] - $b['id'];
                });

                //store divisions in transient for 12 hours
                set_transient($transientName, $requestResponse, 12 * HOUR_IN_SECONDS);
            }

            return $requestResponse;
        }

        protected function getAuth(): \JimsGroup\Facade\Authenticate
        {
            $auth = new \JimsGroup\Facade\Authenticate;
            $auth->setFmsConfig(Jims_Plugin::FMS_TOKENS);

            return $auth;
        }

        protected function getFMSAuthToken()
        {
            $auth = new Authenticate();
            $auth->setFmsConfig(Jims_Plugin::FMS_TOKENS);

            $transientName = StringHelper::GetPrefixed('fms_auth_token_internal');
            if(false === ($cachedToken = get_transient($transientName))){
                [$data, $auth_token] = $auth->CustomerBookingAccessToken($this->getMode());
                $cachedToken = $auth_token;
                set_transient($transientName, $auth_token, 0.1 * HOUR_IN_SECONDS);
            }

            [$data, $auth_token] = $auth->CustomerBookingAccessToken($this->getMode());

            return $cachedToken;
        }

        protected function getMode($default = 'production')
        {
            $settings = OptionHelper::get(NativeFormSettingsPage::$option_name, []);

            if (isset($settings['fms_environment']) && in_array($settings['fms_environment'], Jims_Plugin::FMS_API_MODES)) {
                return $settings['fms_environment'];
            }

            return $default;
        }

        /**
         * @param mixed $countryCode
         * @param mixed $output
         * @return mixed
         * @throws \Exception
         */
        public function filterOutEmptyDivisions(mixed $countryCode, mixed $output): mixed
        {
            //filter out divisions with country
            try {
                $services = $this->getServices(true, $countryCode);
            } catch (\Exception $e) {
                return [];
            }



            $transientName = StringHelper::GetPrefixed('service_groups' . '_' . $countryCode);
            if(false === ($serviceGroups = get_transient($transientName))){

                $serviceGroups = [];
                //group services by division_id
                foreach ($services as $service) {
                    if (!isset($serviceGroups[$service['division_id']])) {
                        $serviceGroups[$service['division_id']] = 0;
                    }
                    $serviceGroups[$service['division_id']]++;
                }

                set_transient($transientName, $serviceGroups, 5 * HOUR_IN_SECONDS);
            }

            return array_filter($output, function ($division) use ($serviceGroups) {
                return $serviceGroups[$division->id] > 0;
            });
        }
    }
}
