<?php

namespace JimsGroup\Apis;

use JimsGroup\Datatypes\AddressType;
use JimsGroup\Datatypes\AllocationRequestType;
use JimsGroup\Datatypes\BookingResponseType;
use JimsGroup\Datatypes\Collection;
use JimsGroup\Datatypes\Division;
use JimsGroup\Datatypes\FinalizeBookingRequestType;
use JimsGroup\Datatypes\InitialBookingType;
use JimsGroup\Datatypes\Service;
use JimsGroup\Datatypes\UserInfo;

/**
 * Class CustomerBooking
 *
 * @package JimsGroup\Apis
 */
class CustomerBooking extends AbstractApi implements IFmsApi, IApi
{

    /**
     * @var string[]
     */
    protected $paths = [
        'initial' => '/api/CustomerBookings/get-initial-booking-request',
        'allocation' => '/api/bookings/allocate-provider-cb',
        'final' => '/api/bookings/booking-job-cb',
        'validate_address' => '/api/address/address-validation/',
    ];

    private $payload = [];


    private $divisions_list = [];


    private $division_services_list = [];

    private $booking_request_id = 0;


    private $utmSource = '';

    /**
     * Refers to the api key, we use in the allocation
     * request, not the auth token on the header
     *
     * @var string $auth_token
     */
    private $auth_token = '';


    /**
     * Authentication token, send in every request
     * as header basic auth
     *
     * @var string $header_auth_token
     */
    private $header_auth_token = '';

    /**
     * @param $params
     * @return mixed
     * @throws \Exception|\InvalidArgumentException
     */
    public function getInitialBookingRequest($params = [])
    {
        //check if header auth token is set
        if (empty($this->getHeaderAuthToken())) {
            throw new \InvalidArgumentException('Header Auth Token is not set');
        }

        $data = $this->getInitialRequestData();
        $params = array_merge($data, $params);
        $url = $this->getEndpointUrl('initial');
        $response = $this->post($url, $params, [
            'Authorization: Bearer ' . $this->getHeaderAuthToken(),
        ]);

        if (!empty($response)) {
//            error_log($response, 0);
            $response = json_decode($response, true);
        }


        try {
            if ($this->hasPayload($response)) {
                $this->booking_request_id = $response['payload']['bookingRequestId'];
                return $this->payload = $response['payload']['jimsServices'];
            }
        } catch (\Exception $e) {
            error_log($e->getMessage(), 0);
            throw new \Exception('Initial Request Payload is not valid');
        }

        return $response;
    }

    /**
     * todo this needs an endpoint on findajim platform to allow using only one google api key
     * @return array|null
     */
    public function validateBookingAddress(AddressType $bookingAddress)
    {
        return null;
    }

    /**
     * @param $division_id
     * @param $serviceIds
     * @param array $bookingAddress
     * @param array $userInfo
     * @return BookingResponseType|bool
     * @throws \RuntimeException
     */
    public function sendAllocationRequest($division_id, $serviceIds, array $bookingAddress, array $userInfo, $authToken): BookingResponseType|bool
    {
        $payload = $this->getAllocationPayload($division_id, $serviceIds, $bookingAddress, $userInfo, $authToken);

        error_log(json_encode($payload), 0);

        if (!empty($payload)) {
            $url = $this->getEndpointUrl('allocation');
            $response = $this->post($url, $payload, [
                'Authorization: Bearer ' . $this->getHeaderAuthToken(),
            ]);

            return new BookingResponseType($response);
        } else {
            throw new \RuntimeException('Payload is empty');
        }
    }

    /**
     * @param $division_id
     * @param $choosedProviderId
     * @param $user_ip_address
     * @param $provider_info
     * @param $user_info
     * @param $bookingConfirmCode
     * @param int $previous_lead_id
     * @param bool $is_contractor
     * @return BookingResponseType
     * @throws \RuntimeException|\JsonException
     */
    public function sendFinalizeBookingRequest($booking_request_id, $division_id, $service_ids, $choosedProviderId, $user_ip_address, $provider_info, $user_info, $bookingConfirmCode, $previous_lead_id = 0, $is_contractor = false): BookingResponseType
    {
        if (empty($this->booking_request_id) || $this->booking_request_id == 0) {
            $this->booking_request_id = $booking_request_id;
        }

        $payload = $this->getFinalizeBookingPayload($this->booking_request_id, $division_id, $service_ids, $choosedProviderId, $user_ip_address, $provider_info, $user_info, $bookingConfirmCode, $previous_lead_id, $is_contractor);

        if (!empty($payload)) {
            $url = $this->getEndpointUrl('final');
            $response = $this->post($url, $payload, [
                'Authorization: Bearer ' . $this->getHeaderAuthToken(),
            ]);
        } else {
            throw new \RuntimeException('Payload is empty');
        }

        return new BookingResponseType($response);
    }

    /**
     * @return Collection
     */
    public function getDivisions()
    {
        $result = [];
        $this->getPayload();

        if (empty($this->divisions_list)) {
            foreach ($this->payload as $service) {
                $result[$service['group_id']] = new Division($service['group_name'], $service['group_id'], $service['group_icon_url']);
            }

            $this->divisions_list = new Collection($result);
        }

        return $this->divisions_list;
    }

    /**
     * @param $url
     * @return mixed|string
     */
    function getDomainNameFromUrl($url)
    {
        $pieces = parse_url($url);
        return isset($pieces['host']) ? $pieces['host'] : '';
    }

    function isUrl($string)
    {
        return preg_match('/^(http|https):\/\/[a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(:[0-9]{1,5})?(\/.*)?$/i', $string);
    }

    /**
     * @return Collection
     */
    public function getDivisionServices($division_id)
    {
        $result = [];
        $this->getPayload();

        if (empty($this->division_services_list)) {
            foreach ($this->payload as $service) {

                if (!isset($result[$service['group_id']])) {
                    $result[$service['group_id']] = [];
                }

                $result[$service['group_id']][] = new Service(
                    $service['id'],
                    $service['name'],
                    $service['short_name'],
                    $service['code'],
                    $service['friendly_name'],
                    $service['goblitz_name'],
                    $service['category_name']
                );
            }

            $this->division_services_list = new Collection($result);
        }

        return $this->division_services_list->find($division_id) ?? new Collection([]);
    }

    /**
     * @return array
     */
    protected function getInitialRequestData(): array
    {
        return (new InitialBookingType())->toArray();
    }

    /**
     * @return mixed
     */
    private function getPayload()
    {
        if (empty($this->payload)) {
            $this->payload = $this->getInitialBookingRequest();
        }

        return $this->payload;
    }

    /**
     * @param $division_id
     * @param $serviceIds
     * @param array $bookingAddress
     * @param array $userInfo
     * @return array
     */
    protected function getAllocationPayload($division_id, $serviceIds, array $bookingAddress, array $userInfo, $apiKey)
    {
        $allocationRequestData = new AllocationRequestType([
            'division_id' => $division_id,
            'selectedServicesIds' => $serviceIds,
            'utmSource' => $this->getUtmSource(),
        ]);

        //add lead note, lead note solves the issue of user notes not appearing in the lead
        if (!empty($userInfo['comments'])) {
            $allocationRequestData->addLeadNote($userInfo['comments']);
        }

        $allocationRequestData->setBookingAddress(...$bookingAddress);
        $allocationRequestData->setUserInfo(...$userInfo);
        $allocationRequestData->setApiKey($apiKey);
        $allocationRequestData->setCountryCode($bookingAddress['country']);


        return $allocationRequestData->toArray();
    }

    /**
     * @param $booking_request_id
     * @param $division_id
     * @param $service_ids
     * @param $choosedProviderId
     * @param $user_ip_address
     * @param $provider_info
     * @param $user_info
     * @param $bookingConfirmCode
     * @param $previous_lead_id
     * @param bool $is_contractor
     * @return array|mixed
     */
    protected function getFinalizeBookingPayload($booking_request_id, $division_id, $service_ids, $choosedProviderId, $user_ip_address, $provider_info, $user_info, $bookingConfirmCode, $previous_lead_id = 0, $is_contractor = false)
    {

        $finalizeBookingData = new FinalizeBookingRequestType([
            'id' => $booking_request_id,
            'allocate_confirm_codes' => $bookingConfirmCode,
            'division_id' => $division_id,
            'choosedProviderId' => $choosedProviderId,
            'ipAddress' => $user_ip_address,
            'selectedServicesIds' => $service_ids,
            'previousLeadId' => $is_contractor ? 0 : $previous_lead_id,
            'utmSource' => $this->getUtmSource(),
        ]);


        if ($previous_lead_id > 0) {
            //this is set to false in order to instruct the API that the recipient is not a previous client
            // if previous client the lead will be updated
            // if not previous client the lead will be created
            $finalizeBookingData->setConfirmedNotPreviousClient(false);
        }

        if (!empty($provider_info)) {

            if ((!empty($provider_info['is_jims_franchisee']) && $provider_info['is_jims_franchisee'] === false) || $is_contractor) {
                //otherServices property is to be CB30_Transfer if is contractor and CB30_Book if is a fse
                //this is set to Jims in order to instruct the API that the recipient is not a Jims Franchisee
                $finalizeBookingData->setOtherServices('CB30_Transfer');
            }


            if ((!empty($provider_info['is_jims_franchisee']) && $provider_info['is_jims_franchisee'] === false) || $is_contractor) {
                //this is set to GB in order to instruct the API that the recipient is not a Jims Franchisee
                $finalizeBookingData->setBrandSource('GB');
            }

//        $finalizeBookingData->setBookingRequestId ( $booking_request_id );
            $finalizeBookingData->setProviderInfo($provider_info);
        } else {
            $finalizeBookingData->setProviderInfo(null);
        }

        $finalizeBookingData->setUserInfo(...$user_info);

        return $finalizeBookingData->toArray();
    }

    /**
     *
     *
     * @param array $response
     * @return bool
     */
    private function hasPayload($response)
    {
        if (!is_array($response)) {
            throw new \Exception('Payload is not valid');
        }

        return isset($response['payload'], $response['payload']['jimsServices']);
    }

    /**
     * @return int
     */
    public function getBookingRequestId(): int
    {
        return $this->booking_request_id;
    }

    /**
     * @return string
     */
    public function getAuthToken(): string
    {
        return $this->auth_token;
    }

    /**
     * @param string $auth_token
     */
    public function setAuthToken(string $auth_token): void
    {
        $this->auth_token = $auth_token;
    }

    /**
     * @return bool
     */
    public function authTokenIsValid(): bool
    {
        return !empty($this->auth_token);
    }

    public function getHeaderAuthToken(): string
    {
        return $this->header_auth_token;
    }

    public function setHeaderAuthToken(string $header_auth_token): void
    {
        $this->header_auth_token = $header_auth_token;
    }

    public function getUtmSource(): string
    {
        return $this->utmSource;
    }

    public function setUtmSource(string $utmSource): void
    {
        $this->utmSource = $utmSource;
    }
}