<?php

namespace JimsGroup\Apis;

use JimsGroup\Datatypes\FMSAuthResponseType;
use JimsGroup\Utilities\HttpRequest;
use JsonException;

/**
 *
 */
class FMSAuthentication extends AbstractApi
{
    /**
     * @var string
     */
    protected $mode = 'production';

    /**
     * Who requests the token
     * @var string
     */
    protected $client_id = 'fms_user';

    /**
     * What credentials to use to get the token
     * @var string
     */
    protected $grant_type = 'password';


    /**
     * @var FMSAuthResponseType|null
     */
    protected $user = null;


    /**
     * @var string
     */
    protected $secret = '';


    protected $token = '';

    /**
     * @var string
     */
    protected $scope = 'openid profile roles jimsApi session';

    protected $api_urls = [
        'production' => 'https://id.jims.net',
        'development' => 'https://id-test.cloud.jims.net',
        'uat' => 'https://id.uat.aws.jims.net',
        'hf' => 'https://id.hf.aws.jims.net',
        'local' => 'http://localhost:3000',
    ];

    protected $paths = [
        'identity' => '/identity/connect/token'
    ];

    public function __construct($secret = '', $mode = 'production')
    {
        $this->secret = $secret;
        $this->mode = $mode;

        parent::__construct($mode);
    }

    /**
     * @param $secret
     * @param $client_id
     * @param $grant_type
     * @param $scope
     * @param $mode
     * @return void
     */
    public function setParams($secret, $client_id, $grant_type, $scope, $mode)
    {
        $this->secret = $secret;
        $this->client_id = $client_id;
        $this->grant_type = $grant_type;
        $this->scope = $scope;

        $this->setMode($mode);
    }

    /**
     * @param $username
     * @param $password
     * @return boolean
     * @throws JsonException
     */
    public function connect($username, $password): bool
    {
        $response = $this->getUserInfo($username, $password);
        if ($response) {
            $this->user = new FMSAuthResponseType($response['payload']);
            return true;
        } else {
            return false;
        }
    }

    /**
     * @return FMSAuthResponseType|null
     */
    public function getUser(): ?FMSAuthResponseType
    {
        return $this->user;
    }


    /**
     * @param $username
     * @param $password
     * @return array|null
     * @throws JsonException
     */
    public function getUserInfo($username, $password): ?array
    {
        if(empty($this->token)) return null;

        [$data, $response] = $this->RequestToken($this->client_id, $this->grant_type, $this->scope, [
            'username' => $username,
            'password' => $password,
        ], $this->token);

        return $this->parseInfo($response);
    }

    /**
     * @param string $scope
     * @return void
     */
    public function setScope(string $scope): void
    {
        $this->scope = $scope;
    }

    /**
     * @param string $secret
     * @return void
     */
    public function setSecret(string $secret): void
    {
        $this->secret = $secret;
    }

    /**
     * @param string $client_id
     * @return void
     */
    public function setClientId(string $client_id): void
    {
        $this->client_id = $client_id;
    }

    /**
     * @param string $grant_type
     * @return void
     */
    public function setGrantType(string $grant_type): void
    {
        $this->grant_type = $grant_type;
    }

    /**
     * @param $username
     * @param $password
     * @return array|null
     * @throws JsonException
     */
    public function RequestToken($client_id, $grant_type, $scope, $extra = [], $token = null): array|null
    {
        $http = new HttpRequest();
        $url = $this->getEndpointUrl('identity');

        $data = array_merge([
            'client_id' => $client_id,
            'grant_type' => $grant_type,
            'scope' => $scope,
            'client_secret' => $this->secret,
        ], $extra);

        //remove empty values
        $data = array_filter($data, function ($value) {
            return !empty($value);
        });

        if (!empty($token)) {
            $http->addHeader('Authorization: Basic ' . $token);
        }

        $response = $http->post($url, $data, 'application/x-www-form-urlencoded');

        if($this->isJSON($response)){
            $parsed_response = \json_decode($response, true, 512, JSON_THROW_ON_ERROR);
            return isset($parsed_response['access_token']) ? [
                $data,
                $parsed_response['access_token']
            ] : [$data, ''];
        }

        return [$data, ''];
    }

    /**
     * @throws JsonException
     * @param $response
     * @return array|null
     */
    public function parseInfo($response): ?array
    {
        //if token is not empty, then we are connected
        if (empty($response)) {
            return null;
        }

        return HttpRequest::parseJWT($response);
    }

    /**
     * @param $string
     * @return bool
     */
    public function isJSON($string)
    {
        if(!is_string($string)) return false;

        json_decode($string);
        return (json_last_error() == JSON_ERROR_NONE);
    }

    public function setToken(string $token): void
    {
        $this->token = $token;
    }
}