<?php

namespace JimsGroup\Datatypes;

use JimsGroup\Utilities\Logger;

/**
 * Class BookingResponseType
 */
class BookingResponseType extends Datatype
{
    const CONFIRM_CODE_BOOKED = 0;

    const CONFIRM_CODE_NO_AVAILABLE_PROVIDER = 1;

    const CONFIRM_CODE_BOOK_TOO_SOON = 2;

    const CONFIRM_CODE_UNSERVICED = 3;

    const CONFIRM_CODE_NOT_KNOWN_ADDRESS = 4;

    const CONFIRM_CODE_FMS_ISSUE = 5;

    const CONFIRM_CODE_OTHER_ERROR = 6;

    const CONFIRM_CODE_TEST_MODE = 7;

    const CONFIRM_CODE_REPEAT_BOOK = 8;

    const CONFIRM_CODE_APPLICATION_ISSUE = 9;

    const CONFIRM_CODE_NO_CODE = 10;

    public static array $status_messages = [
        self::CONFIRM_CODE_BOOKED => 'Success! Your booking is confirmed.',
        self::CONFIRM_CODE_NO_AVAILABLE_PROVIDER => 'We regret to inform you that we currently do not have a local Jim\'s Franchisee in your area who can provide this service.',
        self::CONFIRM_CODE_BOOK_TOO_SOON => 'We\'re sorry, it appears that a service request for this address was submitted within the last 12 hours. If you haven\'t made the booking, please call our Call Centre for any questions or immediate assistance.',
        self::CONFIRM_CODE_UNSERVICED => 'Unfortunately, all our Franchisees for this location are currently unavailable. If a Franchisee does become available, they will be in contact.',
        self::CONFIRM_CODE_NOT_KNOWN_ADDRESS => 'We couldn\'t locate your address, but don\'t worry. Your inquiry has been forwarded to our Call Centre, and they will contact you shortly.',
        self::CONFIRM_CODE_FMS_ISSUE => 'We apologize for the inconvenience, Unfortunately, we cannot process your request at this time. However, your enquiry has been forwarded to our Call Centre and they will contact you shortly.',
        self::CONFIRM_CODE_OTHER_ERROR => 'We apologize for the inconvenience, Unfortunately, we cannot process your request at this time. However, your enquiry has been forwarded to our Call Centre and they will contact you shortly.',
        self::CONFIRM_CODE_TEST_MODE => 'This is a test mode message. No action is required.',
        self::CONFIRM_CODE_REPEAT_BOOK => 'Quote request is completed. You cannot make this request again. Please select other services.',
        self::CONFIRM_CODE_APPLICATION_ISSUE => 'We apologize for the inconvenience, Unfortunately, we cannot process your request at this time. However, your enquiry has been forwarded to our Call Centre and they will contact you shortly.',
        self::CONFIRM_CODE_NO_CODE => 'We apologize for the inconvenience, Unfortunately, we cannot process your request at this time. However, your enquiry has been forwarded to our Call Centre and they will contact you shortly.',
    ];

    protected $default = [
        'result' => false,
        'message' => '',
        'statusCode' => 0,
        'returnType' => 0,
        'bookingRequestId' => 0,
        'confirmCodes' => 0,
        'bookingConfirmCode' => self::CONFIRM_CODE_NO_CODE,
        'payload' => [],
    ];

    protected $empty = true;

    public function __construct($response_string, $ignore_validation = true)
    {
        $this->empty = true;

        if($this->isJson($response_string)){
            $data = $this->getPayloadFromJson($response_string);
            parent::__construct ($data, $ignore_validation);
            $this->setConfirmCodes ();
        }

        if (!empty($this->data)) {
            $this->empty = false;
        } else {
            $this->data = $this->default;
        }
    }

    public function isBookingCompleted(){
        return $this->result === true && $this->returnType === 2;
    }

    public function isBookingFailed(){
        return $this->result === false && $this->returnType === 1;
    }

    /**
     * @return string
     */
    public function getMessage(){
        return $this->message;
    }

    /**
     * @return string
     */
    public function getErrorMessage(){
        if($this->isBookingFailed ()){
            return $this->getMessage ();
        }
        return 'no error message';
    }

    public function status_messages()
    {
        return self::$status_messages;
    }

    public static function get_status_message($code)
    {
        return self::$status_messages[$code];
    }

    /**
     * @return int
     */
    public function getID(){
        return $this->getUnsafeProperty('bookingRequestId', 0);
    }

    public function getPreviousLeadId(){
        return $this->getUnsafeProperty('previousLeadId', 0);
    }

    /**
     * @return array
     */
    public function getPayload(){
        return $this->payload;
    }

    /**
     * @return bool
     */
    public function hasPayload(){
        //check if payload is array and not empty
        return isset($this->data['payload']) && !empty($this->data['payload']);
    }

    public function getPayloadProperty($property){
        if($this->hasPayload ()){
            return isset($this->data['payload'][$property]) ? $this->data['payload'][$property] : null;
        }
        return null;
    }

    /**
     * @return int
     */
    public function getConfirmCodes(){
        //check if payload has either bookingConfirmCode or confirmCodes

        return (int) $this->bookingConfirmCode;
    }

    /**
     * Set the confirm code from the payload
     * @return void
     */
    public function setConfirmCodes(){
        $this->data['bookingConfirmCode'] = isset($this->data['payload'], $this->data['payload']['bookingConfirmCode']) ? $this->data['payload']['bookingConfirmCode'] : $this->bookingConfirmCode;
    }

    /**
     * @return string
     */
    public function getStatusMessage(){
        if($this->getConfirmCodes () >= 0 && $this->getConfirmCodes () <= 8){
            return self::$status_messages[$this->getConfirmCodes ()];
        }
        return 'Connection error';
    }

    /**
     * @return bool
     */
    public function isEmpty(){
        return $this->empty;
    }

    /**
     * @return string
     * @throws \JsonException
     */
    public function __toString()
    {
        $json = json_encode($this->data, JSON_THROW_ON_ERROR);
        return $json === false ? '[]' : $json;
    }

    /**
     * @throws \JsonException
     */
    protected function isJson($string): bool
    {
        json_decode($string, false, 512);
        return (json_last_error() === JSON_ERROR_NONE);
    }

    /*
     * Json decode recursively and return the parent of the object
     * which as the payload
     *
     * return array
     */
    protected function getPayloadFromJson($string){

        $data = json_decode($string, true);

        if (!empty($data) && isset($data['payload'])) {
            return $data;
        }

        foreach ($data as $key => $value){
            if(is_string ($value) && $this->isJson ($value)){
                $data = json_decode($value, true);
            }

            if(isset($data['payload'])){
                break;
            }
        }

        error_log(json_encode($data), 0);
        return $data;
    }

    /**
     * @param $name
     * @param $default
     * @return mixed|null
     */
    protected function getUnsafeProperty($name, $default = null)
    {
        if(isset($this->data[$name])){
            return $this->data[$name];
        }
        elseif (isset($this->data['payload'], $this->data['payload'][$name])){
            return $this->data['payload'][$name];
        }

        return $default;
    }

}